#!/usr/bin/env python3
"""
Windows Service Manager (System Management Tool)

Parameters:
    --list: List all services
    --info SERVICE: Show service details
    --start SERVICE: Start service
    --stop SERVICE: Stop service
    --restart SERVICE: Restart service
    --auto SERVICE: Set to automatic start
    --manual SERVICE: Set to manual start
    --format FORMAT: Output format (table/json)

Requires: win32serviceutil, win32service, tabulate packages and admin privileges
Warning: Modifying critical services may affect system stability
"""

import os
import sys
import argparse
import logging
from typing import List, Dict, Any, Optional
from tabulate import tabulate
import win32serviceutil
import win32service
import win32con
import wmi
import json
from datetime import datetime

class ServiceManager:
    def __init__(self):
        self.logger = logging.getLogger(__name__)
        self.wmi = wmi.WMI()

    def get_services(self) -> List[Dict[str, Any]]:
        """Get list of all services."""
        services = []
        for service in self.wmi.Win32_Service():
            try:
                services.append({
                    'name': service.Name,
                    'display_name': service.DisplayName,
                    'description': service.Description,
                    'state': service.State,
                    'start_mode': service.StartMode,
                    'path': service.PathName,
                    'account': service.StartName,
                    'pid': service.ProcessId,
                    'can_stop': service.AcceptStop,
                    'can_pause': service.AcceptPause
                })
            except Exception as e:
                self.logger.debug(f"Error getting service info: {str(e)}")
                continue
        return services

    def get_service_info(self, service_name: str) -> Dict[str, Any]:
        """Get detailed information about a specific service."""
        try:
            service = self.wmi.Win32_Service(Name=service_name)[0]
            dependencies = self._get_service_dependencies(service_name)
            
            return {
                'name': service.Name,
                'display_name': service.DisplayName,
                'description': service.Description,
                'state': service.State,
                'start_mode': service.StartMode,
                'path': service.PathName,
                'account': service.StartName,
                'pid': service.ProcessId,
                'can_stop': service.AcceptStop,
                'can_pause': service.AcceptPause,
                'dependencies': dependencies,
                'dependent_services': self._get_dependent_services(service_name)
            }
        except Exception as e:
            self.logger.error(f"Error getting service info: {str(e)}")
            return {}

    def start_service(self, service_name: str) -> bool:
        """Start a Windows service."""
        try:
            win32serviceutil.StartService(service_name)
            return True
        except Exception as e:
            self.logger.error(f"Error starting service: {str(e)}")
            return False

    def stop_service(self, service_name: str) -> bool:
        """Stop a Windows service."""
        try:
            win32serviceutil.StopService(service_name)
            return True
        except Exception as e:
            self.logger.error(f"Error stopping service: {str(e)}")
            return False

    def restart_service(self, service_name: str) -> bool:
        """Restart a Windows service."""
        try:
            win32serviceutil.RestartService(service_name)
            return True
        except Exception as e:
            self.logger.error(f"Error restarting service: {str(e)}")
            return False

    def configure_service(self, service_name: str, 
                         start_type: Optional[str] = None,
                         account: Optional[str] = None,
                         password: Optional[str] = None) -> bool:
        """Configure service properties."""
        try:
            config = {}
            
            if start_type:
                start_type_map = {
                    'auto': win32service.SERVICE_AUTO_START,
                    'manual': win32service.SERVICE_DEMAND_START,
                    'disabled': win32service.SERVICE_DISABLED
                }
                config['start_type'] = start_type_map.get(start_type.lower())
                
            if account:
                config['service_account'] = account
                if password:
                    config['password'] = password
                    
            if config:
                win32serviceutil.ChangeServiceConfig(
                    service_name, **config
                )
            return True
            
        except Exception as e:
            self.logger.error(f"Error configuring service: {str(e)}")
            return False

    def _get_service_dependencies(self, service_name: str) -> List[str]:
        """Get list of services that this service depends on."""
        try:
            service = win32serviceutil.SmartOpenService(
                None, service_name, win32con.SERVICE_QUERY_CONFIG
            )
            config = win32service.QueryServiceConfig(service)
            dependencies = []
            if config[7]:  # Index 7 contains dependencies
                for dep in config[7]:
                    if dep.startswith('\\'):
                        dep = dep[1:]  # Remove leading backslash
                    dependencies.append(dep)
            return dependencies
        except Exception as e:
            self.logger.debug(f"Error getting dependencies: {str(e)}")
            return []

    def _get_dependent_services(self, service_name: str) -> List[str]:
        """Get list of services that depend on this service."""
        try:
            service = win32serviceutil.SmartOpenService(
                None, service_name, 
                win32service.SERVICE_ENUMERATE_DEPENDENTS
            )
            dependents = win32service.EnumDependentServices(
                service, win32service.SERVICE_STATE_ALL
            )
            return [svc[0] for svc in dependents]
        except Exception as e:
            self.logger.debug(f"Error getting dependent services: {str(e)}")
            return []

    def display_results(self, data: Any, output_format: str = 'table'):
        """Display results in specified format."""
        if output_format == 'json':
            print(json.dumps(data, indent=4, default=str))
        else:
            self._display_table_format(data)

    def _display_table_format(self, data: Any):
        """Display data in table format."""
        if isinstance(data, list):
            if data and isinstance(data[0], dict):
                headers = data[0].keys()
                table_data = [[str(item.get(h, '')) for h in headers] 
                            for item in data]
                print(tabulate(table_data, headers=headers, tablefmt='grid'))
        elif isinstance(data, dict):
            for key, value in data.items():
                if isinstance(value, (dict, list)):
                    print(f"\n{key}:")
                    if isinstance(value, dict):
                        for k, v in value.items():
                            print(f"  {k}: {v}")
                    else:  # list
                        for item in value:
                            print(f"  {item}")
                else:
                    print(f"{key}: {value}")

def main():
    parser = argparse.ArgumentParser(
        description='Service Manager - Windows service management utility')
    parser.add_argument('--list', action='store_true',
                       help='List all services')
    parser.add_argument('--info',
                       help='Get detailed information about a specific service')
    parser.add_argument('--start',
                       help='Start a service')
    parser.add_argument('--stop',
                       help='Stop a service')
    parser.add_argument('--restart',
                       help='Restart a service')
    parser.add_argument('--configure',
                       help='Configure a service')
    parser.add_argument('--start-type', choices=['auto', 'manual', 'disabled'],
                       help='Set service start type')
    parser.add_argument('--account',
                       help='Set service account')
    parser.add_argument('--password',
                       help='Set service account password')
    parser.add_argument('--format', choices=['table', 'json'],
                       default='table',
                       help='Output format')
    
    args = parser.parse_args()
    
    # Setup logging
    logging.basicConfig(
        level=logging.INFO,
        format='%(asctime)s - %(levelname)s - %(message)s'
    )
    
    try:
        manager = ServiceManager()
        
        if args.list:
            services = manager.get_services()
            manager.display_results(services, args.format)
            
        elif args.info:
            info = manager.get_service_info(args.info)
            manager.display_results(info, args.format)
            
        elif args.start:
            if manager.start_service(args.start):
                print(f"Successfully started service: {args.start}")
            else:
                print(f"Failed to start service: {args.start}")
                
        elif args.stop:
            if manager.stop_service(args.stop):
                print(f"Successfully stopped service: {args.stop}")
            else:
                print(f"Failed to stop service: {args.stop}")
                
        elif args.restart:
            if manager.restart_service(args.restart):
                print(f"Successfully restarted service: {args.restart}")
            else:
                print(f"Failed to restart service: {args.restart}")
                
        elif args.configure:
            if manager.configure_service(
                args.configure, args.start_type, args.account, args.password
            ):
                print(f"Successfully configured service: {args.configure}")
            else:
                print(f"Failed to configure service: {args.configure}")
                
        else:
            parser.print_help()
            
    except Exception as e:
        logging.error(f"An error occurred: {str(e)}")
        return 1
        
    return 0

if __name__ == "__main__":
    exit(main())
